#define TEXT_MAX 8192 /* in bytes */

void init_text(int screen, int x, int y, int w, int h);
void print_all(void);
void hide_cursor(void);
void show_cursor(void);
void cursor_right(void);
void cursor_left(void);
void cursor_down(void);
void cursor_up(void);
void page_down(void);
void page_up(void);
void backspace_char(void);
void insert_char(int c);
int read_file(const char* file_name);
int write_file(const char* file_name);
